==> cryptide-1.1.0.tgz: package/src/bitcoin.js <==
// Tide Protocol - Infrastructure for the Personal Data economy
// Copyright (C) 2019 Tide Foundation Ltd
// 
// This program is free software and is subject to the terms of 
// the Tide Community Open Source License as published by the 
// Tide Foundation Limited. You may modify it and redistribute 
// it in accordance with and subject to the terms of that License.
// This program is distributed WITHOUT WARRANTY of any kind, 
// including without any implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE.
// See the Tide Community Open Source License for more details.
// You should have received a copy of the Tide Community Open 
// Source License along with this program.
// If not, see https://tide.org/licenses_tcosl-1-0-en

import BigInt from 'big-integer';
import bs58 from 'bs58';
import hash from './hash.js';

const EC = require('elliptic').ec;
const ec = new EC('secp256k1');

const p = BigInt("0fffffffffffffffffffffffffffffffffffffffffffffffffffffffefffffc2f", 16);
const n = BigInt('0fffffffffffffffffffffffffffffffebaaedce6af48a03bbfd25e8cd0364141', 16);
const b = BigInt(7);

export default class bitcoin {
    static formatSign(m, r, s, address) {
        for (var i = 0; i < 4; i++) {
            if (!bitcoin.verifySign(m, r, s, address, i)) {
                continue;
            }

            var data = [27 + i].concat(r.toArray(256).value).concat(s.toArray(256).value);
            return Buffer.from(data).toString('base64');
        }

        return false;
    }

    static verifySign(m, r, s, address, recid) {
        var Rx = BigInt(recid).divide(2).times(n).add(r);
        var RySquared = Rx.times(Rx).times(Rx).add(b).mod(p);
        var RyTag = RySquared.modPow(p.add(1).divide(4), p);
        var Ry = RyTag.subtract(recid).isEven() ? RyTag : p.subtract(RyTag);

        var R = ec.curve.point(Rx.toString(16), Ry.toString(16));
        var rInv = Rx.modInv(n);

        var Rs = R.mul(s.toString(16));
        var Gm = ec.curve.g.mul(n.subtract(m).toString(16));
        var pub = Rs.add(Gm).mul(rInv.toString(16));

        return address === getAddress(pub.getX().toArray(true), pub.getY().toArray(true));
    }

    static getM(message) {
        var b = msg_bytes("Bitcoin Signed Message:\n").concat(msg_bytes(message));
        return BigInt.fromArray(Array.from(hash.shaBuffer(hash.shaBuffer(b))), 256, false).mod(n);
    }

    static getAddress(x, y) {
        return getAddress(x.toArray(256).value, y.toArray(256).value);
    }

    static getAddressFromShares(shrs) {
        var y = shrs.map(shri => ec.curve.point(shri.xi.toString(16), shri.yi.toString(16)))
            .reduce((sum, yi) => sum.add(yi), ec.curve.point(null, null));
        
        return getAddress(y.getX().toArray(true), y.getY().toArray(true))
    }
}

function getAddress(x, y) {
    var body = [0x04].concat(x).concat(y);
    var head = [0x00].concat(Array.from(hash.ripeBuffer(hash.shaBuffer(body))));

    var checkSum = Array.from(hash.shaBuffer(hash.shaBuffer(head))).slice(0, 4)
    return bs58.encode(Buffer.from(head.concat(checkSum)));
}

function msg_numToVarInt(i) {
    if (i < 0xfd) {
        return [i];
    } else if (i <= 0xffff) {
        return [0xfd, i & 255, i >>> 8];
    } else if (i <= 0xffffffff) {
        return [0xfe, i & 255, (i >>> 8) & 255, (i >>> 16) & 255, i >>> 24];
    } else {
        throw ("message too large");
    }
}

function msg_bytes(message) {
    var b = Array.from(Buffer.from(message, 'utf-8'));
    return msg_numToVarInt(b.length).concat(b);
}

==> cryptide-1.5.1.tgz: package/src/bitcoin.js <==
// Tide Protocol - Infrastructure for the Personal Data economy
// Copyright (C) 2019 Tide Foundation Ltd
// 
// This program is free software and is subject to the terms of 
// the Tide Community Open Source License as published by the 
// Tide Foundation Limited. You may modify it and redistribute 
// it in accordance with and subject to the terms of that License.
// This program is distributed WITHOUT WARRANTY of any kind, 
// including without any implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE.
// See the Tide Community Open Source License for more details.
// You should have received a copy of the Tide Community Open 
// Source License along with this program.
// If not, see https://tide.org/licenses_tcosl-1-0-en

import BigInt from 'big-integer';
import bs58 from 'bs58';
import hash from './hash.js';
import EcPoint from './ecPoint';
import { b, n, p } from './eccParams';

export default class bitcoin {
    /**
     * 
     * @param {bigInt.BigInteger} m 
     * @param {bigInt.BigInteger} r 
     * @param {bigInt.BigInteger} s 
     * @param {string} address
     * @returns {(string|boolean)}
     */
    static formatSign(m, r, s, address) {
        for (var i = 0; i < 4; i++) {
            if (!bitcoin.verifySign(m, r, s, address, i)) {
                continue;
            }

            var data = [27 + i].concat(r.toArray(256).value).concat(s.toArray(256).value);
            return Buffer.from(data).toString('base64');
        }

        return false;
    }

    static verifySign(m, r, s, address, recid) {
        var Rx = BigInt(recid).divide(2).times(n).add(r);
        var RySquared = Rx.times(Rx).times(Rx).add(b).mod(p);
        var RyTag = RySquared.modPow(p.add(1).divide(4), p);
        var Ry = RyTag.subtract(recid).isEven() ? RyTag : p.subtract(RyTag);

        var R = new EcPoint(Rx, Ry);
        var rInv = Rx.modInv(n);

        var Rs = R.times(s);
        var Gm = EcPoint.g.times(n.subtract(m));
        var pub = Rs.add(Gm).times(rInv);

        return address === getAddress(pub);
    }

    static getM(message) {
        var b = msg_bytes("Bitcoin Signed Message:\n").concat(msg_bytes(message));
        return BigInt.fromArray(Array.from(hash.shaBuffer(hash.shaBuffer(b))), 256, false).mod(n);
    }

    static getAddress(point) {
        return getAddress(point);
    }
}

function getAddress(point) {
    var body = [0x04].concat(point.toArray());
    var head = [0x00].concat(Array.from(hash.ripeBuffer(hash.shaBuffer(body))));

    var checkSum = Array.from(hash.shaBuffer(hash.shaBuffer(head))).slice(0, 4)
    return bs58.encode(Buffer.from(head.concat(checkSum)));
}

function msg_numToVarInt(i) {
    if (i < 0xfd) {
        return [i];
    } else if (i <= 0xffff) {
        return [0xfd, i & 255, i >>> 8];
    } else if (i <= 0xffffffff) {
        return [0xfe, i & 255, (i >>> 8) & 255, (i >>> 16) & 255, i >>> 24];
    } else {
        throw ("message too large");
    }
}

function msg_bytes(message) {
    var b = Array.from(Buffer.from(message, 'utf-8'));
    return msg_numToVarInt(b.length).concat(b);
}
